@@ -30,8 +30,8 @@ import siputils
 
 
 # Initialise the globals.
-sip_version = 0x04130c
-sip_version_str = "4.19.12"
+sip_version = 0x041310
+sip_version_str = "4.19.16"
 py_version = sys.hexversion >> 8
 py_platform = sys.platform
 plat_py_site_dir = None
@@ -46,6 +46,7 @@ sip_bin_dir = ''
 sip_inc_dir = ''
 sip_root_dir = ''
 sip_module_dir = ''
+sip_module_dest_dir = ''
 sip_sip_dir = ''
 pyi_dir = ''
 sysroot = ''
@@ -154,7 +155,8 @@ def set_build_platform():
         "sunos5":       "solaris-cc",
         "ultrix":       "ultrix-g++",
         "unix_sv":      "unixware-g++",
-        "unixware":     "unixware-cc"
+        "unixware":     "unixware-cc",
+        "haiku1":       "haiku-g++"
     }
 
     build_platform = "none"
@@ -185,7 +187,7 @@ def inform_user():
         siputils.inform("The sip.h header file will be installed in %s." % sip_inc_dir)
 
     if not opts.no_module:
-        siputils.inform("The %s module will be installed in %s." % (sip_module_name, sip_module_dir))
+        siputils.inform("The %s module will be installed in %s." % (sip_module_name, sip_module_dest_dir))
 
     if opts.pyi:
         siputils.inform("The sip.pyi stub file will be installed in %s." % pyi_dir)
@@ -302,30 +304,35 @@ def create_makefiles(macros):
     cfg.set_build_macros(macros)
 
     all_installs = []
-    installs = []
+    top_installs = []
+    gen_installs = []
     subdirs = []
 
     if not opts.no_tools:
         subdirs.append('sipgen')
-        installs.append(
+        top_installs.append(
                 (["sipconfig.py", os.path.join(src_dir, "sipdistutils.py")],
                         cfg.sip_root_dir))
+        gen_installs.append(
+                (os.path.join(src_dir, "siplib", "sip.h"), cfg.sip_inc_dir))
 
     if not opts.no_module:
         subdirs.append('siplib')
 
-    all_installs += installs
+    all_installs.extend(top_installs)
+    all_installs.extend(gen_installs)
 
     # The command to run to generate the dist-info directory.
     mk_distinfo = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                 'mk_distinfo.py')
-    distinfo_dir = os.path.join(cfg.sip_root_dir,
+    distinfo_dir = os.path.join(cfg.sip_module_dir,
             '%s-%s.dist-info' % (sip_module_name.replace('.', '_'),
                     sip_version_str))
-    run_mk_distinfo = '%s %s "$(DESTDIR)" %s installed.txt' % (sys.executable,
-            mk_distinfo, distinfo_dir)
 
     if opts.use_qmake:
+        run_mk_distinfo = '%s %s \\\"$(INSTALL_ROOT)\\\" %s installed.txt' % (
+                sys.executable, mk_distinfo, distinfo_dir)
+
         sipconfig.inform("Creating top level .pro file...")
 
         pro = open("sip.pro", "w")
@@ -333,8 +340,9 @@ def create_makefiles(macros):
         pro.write("TEMPLATE = subdirs\n")
         pro.write("SUBDIRS = %s\n" % " ".join(subdirs))
 
-        if installs:
-            files, path = installs
+        if top_installs:
+            # There will only be one element.
+            files, path = top_installs[0]
             pro.write("\n")
             pro.write("build_system.files = %s\n" % " ".join(files))
             pro.write("build_system.path = %s\n" % quote(path))
@@ -343,22 +351,25 @@ def create_makefiles(macros):
         if opts.distinfo:
             pro.write("\n")
             pro.write("distinfo.extra = %s\n" % run_mk_distinfo)
-            pro.write("distinfo.path = %s\n" % quote(cfg.sip_root_dir))
+            pro.write("distinfo.path = %s\n" % quote(cfg.sip_module_dir))
             pro.write("INSTALLS += distinfo\n")
 
         pro.close()
     else:
+        run_mk_distinfo = '%s %s "$(DESTDIR)" %s installed.txt' % (
+                sys.executable, mk_distinfo, distinfo_dir)
+
         sipconfig.inform("Creating top level Makefile...")
 
         # Note that mk_distinfo.py won't exist if we are building from the
         # repository.
         if opts.distinfo and os.path.isfile(mk_distinfo):
-            installs.append((run_mk_distinfo, None))
+            top_installs.append((run_mk_distinfo, None))
 
         sipconfig.ParentMakefile(
             configuration=cfg,
             subdirs=subdirs,
-            installs=installs
+            installs=top_installs
         ).generate()
 
     if opts.use_qmake:
@@ -390,6 +401,14 @@ def create_makefiles(macros):
         pro.write("HEADERS = %s\n" % " ".join(
                 [qmake_quote(h) for h in headers]))
 
+        if gen_installs:
+            # There will only be one element.
+            files, path = gen_installs[0]
+            pro.write("\n")
+            pro.write("sip_h.files = %s\n" % " ".join(files))
+            pro.write("sip_h.path = %s\n" % quote(path))
+            pro.write("INSTALLS += sip_h\n")
+
         pro.close()
     else:
         sipconfig.inform("Creating sip code generator Makefile...")
@@ -399,6 +418,7 @@ def create_makefiles(macros):
             build_file=os.path.join(src_dir, "sipgen", "sipgen.sbf"),
             dir="sipgen",
             install_dir=os.path.dirname(cfg.sip_bin),
+            installs=gen_installs,
             console=1,
             warnings=1,
             universal=opts.universal,
@@ -406,7 +426,7 @@ def create_makefiles(macros):
             deployment_target=opts.deployment_target
         ).generate()
 
-    # The code generator installs.
+    # The implied code generator installs.
     if not opts.no_tools:
         sip_dir, sip_exe = os.path.split(cfg.sip_bin)
         if sys.platform == 'win32':
@@ -417,18 +437,18 @@ def create_makefiles(macros):
     # The module installs.
     module_installs=[]
 
-    if not opts.no_tools:
-        module_installs.append(
-                (os.path.join(src_dir, "siplib", "sip.h"), cfg.sip_inc_dir))
-
     if opts.pyi:
         module_installs.append((os.path.join(src_dir, 'sip.pyi'), pyi_dir))
 
     all_installs.extend(module_installs)
 
     if not opts.no_module:
-        mod_ext = '.pyd' if sys.platform == 'win32' else '.so'
-        all_installs.append(('sip' + mod_ext, cfg.sip_module_dir))
+        if sys.platform == 'win32':
+            mod = 'sip.lib' if opts.static else 'sip.pyd'
+        else:
+            mod = 'libsip.a' if opts.static else 'sip.so'
+
+        all_installs.append((mod, sip_module_dest_dir))
 
     if opts.use_qmake:
         sipconfig.inform("Creating sip module .pro file...")
@@ -453,7 +473,11 @@ def create_makefiles(macros):
 
         if sip_module_name != 'sip':
             pro.write("\n")
-            pro.write('DEFINES += SIP_MODULE_NAME=\\\\\\"%s\\\\\\"\n' % sip_module_name)
+            pro.write('DEFINES += SIP_MODULE_NAME=%s\n' % sip_module_name)
+
+            base_name = sip_module_name.split('.')[-1]
+            if base_name != 'sip':
+                pro.write('DEFINES += SIP_MODULE_BASENAME=%s\n' % base_name)
 
         if not opts.static:
             # These only need to be correct for Windows.
@@ -485,7 +509,7 @@ target.files = $$PY_MODULE
 """ % (debug_suffix, link_lib_dir))
 
         pro.write("\n")
-        pro.write("target.path = %s\n" % cfg.sip_module_dir)
+        pro.write("target.path = %s\n" % sip_module_dest_dir)
         pro.write("INSTALLS += target\n")
 
         if opts.pyi:
@@ -494,12 +518,6 @@ target.files = $$PY_MODULE
             pro.write("sip_pyi.path = %s\n" % pyi_dir)
             pro.write("INSTALLS += sip_pyi\n")
 
-        if not opts.no_tools:
-            pro.write("\n")
-            pro.write("sip_h.files = sip.h\n")
-            pro.write("sip_h.path = %s\n" % cfg.sip_inc_dir)
-            pro.write("INSTALLS += sip_h\n")
-
         c_sources = get_sources("siplib", "*.c")
         cpp_sources = get_sources("siplib", "*.cpp")
         pro.write("\n")
@@ -521,7 +539,7 @@ target.files = $$PY_MODULE
             configuration=cfg,
             build_file=os.path.join(src_dir, "siplib", "siplib.sbf"),
             dir="siplib",
-            install_dir=cfg.sip_module_dir,
+            install_dir=sip_module_dest_dir,
             installs=module_installs,
             console=1,
             warnings=1,
@@ -533,8 +551,11 @@ target.files = $$PY_MODULE
         )
 
         if sip_module_name != 'sip':
-            makefile.DEFINES.append(
-                    'SIP_MODULE_NAME=\\"%s\\"' % sip_module_name)
+            makefile.DEFINES.append('SIP_MODULE_NAME=%s' % sip_module_name)
+
+            base_name = sip_module_name.split('.')[-1]
+            if base_name != 'sip':
+                makefile.DEFINES.append('SIP_MODULE_BASENAME=%s' % base_name)
 
         if src_dir != build_dir:
             src_siplib_dir = os.path.join(src_dir, "siplib")
@@ -908,7 +929,7 @@ def main(argv):
     global plat_bin_dir, plat_py_conf_inc_dir, plat_py_inc_dir
     global plat_py_lib_dir, plat_py_site_dir, plat_sip_dir
     global sip_bin_dir, sip_inc_dir, sip_root_dir, sip_module_dir, sip_sip_dir
-    global pyi_dir
+    global sip_module_dest_dir, sip_module_name, pyi_dir
 
     # Set defaults.
     sip_bin_dir = plat_bin_dir
@@ -941,7 +962,9 @@ def main(argv):
     if opts.destdir is not None:
         sip_root_dir = opts.destdir
 
-    global sip_module_name
+    # The module directory might have been set in a configuration file.
+    if not sip_module_dir:
+        sip_module_dir = sip_root_dir
 
     sip_module_name = opts.sip_module
 
@@ -949,10 +972,10 @@ def main(argv):
 
     if len(module_path) > 1:
         del module_path[-1]
-        module_path.insert(0, sip_root_dir)
-        sip_module_dir = os.path.join(*module_path)
+        module_path.insert(0, sip_module_dir)
+        sip_module_dest_dir = os.path.join(*module_path)
     else:
-        sip_module_dir = sip_root_dir
+        sip_module_dest_dir = sip_module_dir
 
     # Override from the command line.
     if opts.platform is not None:
@@ -970,7 +993,7 @@ def main(argv):
     if opts.pyidir is not None:
         pyi_dir = opts.pyidir
     else:
-        pyi_dir = sip_module_dir
+        pyi_dir = sip_module_dest_dir
 
     # Get the platform specific macros for building.
     macros = siputils.parse_build_macros(
